#!/usr/bin/env python 
import setpath
import compiler.ast

from bike.testutils import createSourceNodeAt
from bike import testdata
import unittest
from bike.parsing.extended_ast import *

class PackageContainerTests:
    def testGetPackage_throwsNoSuchPackageExceptionIfPackageDoesntExist(self):
        self.target.packages = []
        try:
            self.target.getPackage("doesntexist")
            self.fail("should have thrown exception")
        except NoSuchPackageException:
            pass
            
    def testGetPackage_returnsNamedPackage(self):
        self.target.packages = [Package("foo","foo","")]
        assert self.target.getPackage("foo").name == "foo"

    def testAddPackge_raisesPackageAlreadyExistsExceptionIfPackageAlreadyExists(self):
        self.target.addPackage(Package("foo","foo",""))
        try:
            self.target.addPackage(Package("foo","foo",""))
            self.fail("should have thrown exception")
        except PackageAlreadyExistsException:
            pass

    def testAddPackage_addsThePackage(self):
        p = Package("foo","foo")
        self.target.addPackage(p)
        assert self.target.getPackage("foo") == p

        

class SourceContainerTests:
    def testGetSource_throwsNoSuchModuleExceptionIfSourceDoesntExist(self):
        self.target.packages = []
        try:
            self.target.getSource("doesntexist")
            self.fail("should have thrown exception")
        except NoSuchModuleException:
            pass
            
    def testGetSource_returnsNamedSource(self):
        self.target.modules = [Source.createFromString("foo","foo","hello")]
        assert self.target.getSource("foo").modulename == "foo"

    def testAddPackge_raisesModuleAlreadyExistsExceptionIfSourceAlreadyExists(self):
        self.target.addSource(Source.createFromString("foo","foo","hello"))
        try:
            self.target.addSource(Source.createFromString("foo","foo","hello"))
            self.fail("should have thrown exception")
        except ModuleAlreadyExistsException:
            pass

    def testAddPackage_addsThePackage(self):
        p = Package("foo","foo",[],[])
        self.target.addPackage(p)
        assert self.target.getPackage("foo") == p


class TestRoot(unittest.TestCase, PackageContainerTests):
    def setUp(self):
        self.target = Root()

    def test_addPackage_setsTheFQNAndPackageScope(self):
        p = Package("bah","bah")
        self.target.addPackage(p)
        assert self.target.getPackage("bah").fqn == "bah"
        assert self.target.getPackage("bah").parentScope == self.target

    def test_addSource_setsTheFQNAndPackageScope(self):
        s = Source.createFromString("bah.py","bah","class TheClass: pass")
        self.target.addSource(s)
        assert s.fqn == "bah"


class TestPackage(unittest.TestCase, PackageContainerTests,SourceContainerTests):
    def setUp(self):
        root = Root()
        self.target = Package("foo","foo",root)
        root.addPackage(self.target)

    def test_addPackage_setsTheFQNAndPackageScope(self):
        p = Package("bah","bah")
        self.target.addPackage(p)
        assert self.target.getPackage("bah").fqn == "foo.bah"
        assert self.target.getPackage("bah").parentScope == self.target

    def test_addSource_setsTheFQNAndPackageScope(self):
        s = Source.createFromString("bah.py","bah","class TheClass: pass")
        self.target.addSource(s)
        assert s.fqn == "foo.bah"


class TestSource(unittest.TestCase):
    def testCreatesAbsolutePath(self):
        s = Source.createFromString("foo.py","foo","")
        assert s.filename == os.path.abspath("foo.py")        

if __name__ == "__main__":
    unittest.main()
